﻿var crudProductBaseUrl = "Product";

var GRID_PAGE_SIZE = 10
var AUTO_COMPLETE_PAGE_SIZE = 8

var GRID_MAIN_HEIGHT = 500

var ID = {
    GridMain: "gridProduct",
    FormMain: "formMain",
    FormPopup: "popupTemplate",
    CommandTemplate: "commandTemplate"
};

var COLUMNS_GRID_MAIN = {
    ProductID: { Name: "ProductID", Width: 50 },
    ProductName: { Name: "ProductName", Width: 170 },
    Description: { Name: "Description", Width: 190 },
    LogonURL: { Name: "LogonURL", Width: 210 },
    IsInternal: { Name: "IsInternal", Width: 65 }
};

$(document).ready(function () {
    //add kendo validator to page
    window.validator = $("#" + ID.FormMain).kendoValidator().data(KENDO_DATA.Validator);

    var gridDataSource = new kendo.data.DataSource({
        transport: {
            read: {
                url: crudProductBaseUrl + "/ProductList/?IncludeInternal=True",
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                type: "post"
            },
            update: {
                url: crudProductBaseUrl + "/Update",
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                type: "post"
            },
            destroy: {
                url: crudProductBaseUrl + "/Delete",
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                type: "post"
            },
            create: {
                url: crudProductBaseUrl + "/Add",
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                type: "post"
            },

            parameterMap: function (options, operation) {
                if (operation !== "read" && options.models) {
                    return JSON.stringify(options);
                }
            }
        },
        batch: true,
        pageSize: GRID_PAGE_SIZE,
        schema: {
            model: {
                id: COLUMNS_GRID_MAIN.ProductID.Name,
                fields: {
                    ProductID: { type: "guid", editable: false, nullable: true },
                    ProductName: {
                        type: "string",
                        editable: true,
                        validation: { //set validation rules
                            unique: function (input) {
                                var msg = null;

                                if (input.is("[id=" + COLUMNS_GRID_MAIN.ProductName.Name + "]")) {
                                    var dataSource = $("#" + ID.GridMain).data(KENDO_DATA.Grid).dataSource;
                                    var dataUrl = dataSource.options.transport.read.url;

                                    var ds = new kendo.data.DataSource({
                                        transport: {
                                            read: {
                                                url: dataUrl,
                                                async: false
                                            }
                                        },
                                        error: function (e) {
                                            msg = e.Message;
                                            input.attr("data-unique-msg", msg)
                                        }
                                    });

                                    //Apply filter
                                    ds.filter({
                                        logic: "or",
                                        filters: [
                                            { field: COLUMNS_GRID_MAIN.ProductName.Name, operator: "eq", value: input.val() }
                                        ]
                                    });

                                    if (ds.total() != 0)
                                        msg = "The Product must be unique.";

                                    if (msg != null)
                                        input.attr("data-unique-msg", msg);
                                }
                                return (msg == null);
                            }
                        }
                    },
                    Description: { type: "string", editable: true },
                    LogonURL: { type: "string", editable: true },
                    IsInternal: { type: "boolean", editable: true }
                }
            }
        },
        error: function (e) {
            alert("Error in grid data source");
        }
    });

    $("#" + ID.GridMain).kendoGrid({
        dataSource: gridDataSource,
        height: GRID_MAIN_HEIGHT,
        pageable: true,
        toolbar: [{ name: "create", text: "Add Product" }],
        columns: [
            { field: COLUMNS_GRID_MAIN.ProductName.Name, title: "Product", width: COLUMNS_GRID_MAIN.ProductName.Width, attributes: { style: "white-space: nowrap" } },
            { field: COLUMNS_GRID_MAIN.Description.Name, title: "Description", width: COLUMNS_GRID_MAIN.Description.Width, attributes: { style: "white-space: nowrap" } },
            { field: COLUMNS_GRID_MAIN.LogonURL.Name, title: "Logon URL", width: COLUMNS_GRID_MAIN.LogonURL.Width, attributes: { style: "white-space: nowrap" } },
            { field: COLUMNS_GRID_MAIN.IsInternal.Name, title: "Internal", width: COLUMNS_GRID_MAIN.IsInternal.Width, attributes: { style: "white-space: nowrap; text-align: center;" }, template: '<input type="checkbox" #= IsInternal ? checked="checked" : "" # disabled="disabled" />' },
            { template: kendo.template($("#" + ID.CommandTemplate).html()), title: "&nbsp;", width: 140 }
            //{ command: ["edit", "destroy"], title: "&nbsp;", width: 140 }
        ],
        editable: {
            mode: "popup",
            template: $("#" + ID.FormPopup).html(),
            confirmation: "Are you sure you want to remove this product?"
        },
        edit: function (e) { return PopupSetup(e); },
        save: function (e) { return PopupSave(e); },
        filterable: true,
        sortable: true,
        selectable: "row single",
        scrollable: true,
        columnMenu: true,
        resizable: true
    });

    GridColumnHeaderSetup($("#" + ID.GridMain));
    GridAddSetup($("#" + ID.GridMain));
})

function GridColumnHeaderSetup(grid) {
    //Loop through column headers and remove href
    var headers = grid.find(".k-grid-header-wrap th");
    headers.each(function () {
        var $th = $(this);
        if ($th.find(".k-link").attr("href") != null)
            $th.find(".k-link").removeAttr("href");
    });
}

function GridAddSetup(grid) {
    //Align Add button to the right
    grid.find(".k-grid-add").css("float", "right");
    //Remove href
    if (grid.find(".k-grid-add").attr("href") != null)
        grid.find(".k-grid-add").removeAttr("href")
}

// Event Handler for popup Edit event
function PopupSetup(e) {
    //Reference to popup container
    var editor = e.container;
    //Determine if Add or Edit
    var isEdit = ((e.model.ProductID != null) && (e.model.ProductID.length != 0));

    PopupSetTitle(editor, isEdit);
    PopupSetButtons(editor);
    PopupSetFieldKey(editor, isEdit);
    //Hide or Show edit fields
    PopupSetFieldEditItems(isEdit);
    //Set activite event handler
    PopupBindEvent(editor, "activate", function (e) { return PopupActivate(e, isEdit) });
    //Center popup window
    PopupCenter(editor);
}

// Event Handler for popup activation
function PopupActivate(e, isEdit) {
    //Reference to popup container
    var editor = $("div.k-widget.k-window").last();
    if (isEdit) {
        //Place focus on description field
        editor.find("#" + COLUMNS_GRID_MAIN.Description.Name).select();
    }
    else {
        //Place focus on product name field
        editor.find("#" + COLUMNS_GRID_MAIN.ProductName.Name).select();
    }
}

// Event Handler for popup Save event
function PopupSave(e) {
    //Reference to popup container
    var editor = e.container;
    //Determine if Add or Edit popup
    var isEdit = ((e.model.ProductID != null) && (e.model.ProductID.length != 0));
    if (!isEdit) {
        if (e.model.Description != editor.find("#" + COLUMNS_GRID_MAIN.Description.Name).val()) {
            e.model.Description = editor.find("#" + COLUMNS_GRID_MAIN.Description.Name).val();
            e.model.dirty = true;
        }
        if (e.model.LogonURL != editor.find("#" + COLUMNS_GRID_MAIN.LogonURL.Name).val()) {
            e.model.LogonURL = editor.find("#" + COLUMNS_GRID_MAIN.LogonURL.Name).val();
            e.model.dirty = true;
        }
    }
}

// Event Handler for auto complete selection
var onAutoCompleteSelect = function (e) {
    var editor = $("div.k-widget.k-window").last();
    var ProductName = editor.find("#" + COLUMNS_GRID_MAIN.ProductName.Name).val().toUpperCase();
    for (var prod = 0; prod < e.sender.dataSource.view().length; prod++)
    {
        if (e.sender.dataSource.view()[prod].ProductName.toUpperCase() == ProductName) {
            editor.find("#" + COLUMNS_GRID_MAIN.Description.Name)[0].value = e.sender.dataSource.view()[prod].Description;
            editor.find("#" + COLUMNS_GRID_MAIN.LogonURL.Name)[0].value = e.sender.dataSource.view()[prod].LogonURL;
            break;
        }
    }
};

function PopupSetTitle(editor, isEdit) {
    //Set appropriate popup title
    if (isEdit)
        editor.kendoWindow("title", "Edit Product");
    else
        editor.kendoWindow("title", "Add Product");
}

function PopupSetButtons(editor) {
    //Change 'Update' button text to 'Save'
    editor.find("a.k-button.k-button-icontext.k-grid-update").html('<span class="k-icon k-update"></span>Save');
    //align buttons on the right edge
    editor.find(".k-grid-cancel").remove();
    editor.find(".k-grid-update").css("float", "right");
    if (editor.find(".k-grid-update").attr("href") != null)
        editor.find(".k-grid-update").removeAttr("href")
}

//Set up Key field for adding a tenant
function PopupSetFieldKey(editor, isEdit) {
    var fieldID = COLUMNS_GRID_MAIN.ProductName.Name;
    var autoCompleteDatasource = new kendo.data.DataSource({
        serverFiltering: true,
        transport: {
            read: {
                url: crudProductBaseUrl + "/AutoCompleteProductList"
            },
            parameterMap: function (data) {
                return { ProdName: $("#" + fieldID).val() };
            }
        },
        error: function (e) {
            alert("Unable to get Product List.");
        }
    });

    var autoCompleteField = editor.find("#" + fieldID).kendoAutoComplete({
        minLength: 1,
        dataTextField: fieldID,
        dataValueField: fieldID,
        suggest: true,
        ignoreCase: true,
        pageSize: AUTO_COMPLETE_PAGE_SIZE,
        select: onAutoCompleteSelect,
        dataSource: autoCompleteDatasource,
        schema: {
            model: {
                id: fieldID,
                fields: {
                    ProductName: { editable: false },
                    Description: { editable: true },
                    LogonURL: { editable: true }
                }
            },
        }
    }).data(KENDO_DATA.AutoComplete);

    if (isEdit)
        autoCompleteField.enable(false);
    else
        autoCompleteField.enable(true);
}

function PopupSetFieldEditItems(isEdit) {
    //Use to hide edit fields when Add Tenant popup displayed and show edit fields when Edit popup displayed
    if (!isEdit) {
        $("tr.edit").hide();
        $("fieldset.edit").hide();
    }
    else {
        $("tr.edit").show();
        $("fieldset.edit").show();
    }
}

function PopupBindEvent(editor, eventName, func) {
    //Need to set focus when popup activated
    editor.data(KENDO_DATA.Window).bind(eventName, func);
}

function PopupCenter(editor) {
    editor.data(KENDO_DATA.Window).center();
}
